local screen = {guiGetScreenSize()}
local x, y = (screen[1]/1366), (screen[2]/768)

local panelState = false

local activeButton = false
local buttons = false

local camera = false
local lastCursorPos = false
local lastPlayerPos = false

local fullScreenMode = false

local showroomPed = false

local barberID = false

local barber = false

-- GENERAL FUNCTIONS

function togglePanel(state)
    if (state) then
        if (not panelState) then
            barber = {}

            initBarber()

            barber.font = {}
            barber.font[1] = dxCreateFont('assets/fonts/regular.ttf', y * 12)

            barber.colorSelected = 0
            barber.typeSelected = 1

            addEventHandler('onClientRender', root, onRenderHandler)
            addEventHandler('onClientClick', root, onClickHandler)
            
            setTimer(
                function()
                    addEventHandler('onClientKey', root, onKeyHandler, true, 'low-500')
                end, 1000, 1
            )
        end
    else
        if (panelState) then
            fadeCamera(false, 1)
            
            tick = false
            
            setTimer(
                function()
                    removeEventHandler('onClientRender', root, onRenderHandler)
                    removeEventHandler('onClientClick', root, onClickHandler)
                    removeEventHandler('onClientKey', root, onKeyHandler)
                    
                    resetCamera()
                    
                    if (isElement(showroomPed)) then destroyElement(showroomPed) end
                    showroomPed = false
                    
                    barber = false
                    
                    tick = false

                    fadeCamera(true, 1)
                end, 1000, 1
            )
        end
    end

    setPlayerHudComponentVisible('radar', not state)
    showCursor(state)
    showChat(not state)

    panelState = state
end

function initBarber()
    fadeCamera(false, 1)
    
    local barber = BARBERS[barberID]
    
    setTimer(
        function(barber)
            setElementFrozen(localPlayer, true)

            addEventHandler('onClientPreRender', root, onPreRenderHandler)

            local x, y, z = unpack(barber.Preview)
            local rz = 0

            showroomPed = createPed(0, x, y, z)

            for i = 2, 17 do
                if (getPedClothesID(localPlayer, i)) then
                    setPedClothes(showroomPed, i, getPedClothesID(localPlayer, i))
                end
            end

            setPedClothes(showroomPed, 1, HAIRS.Types[1])

            setElementRotation(showroomPed, 0, 0, barber['Preview'].Rotation)
            setElementInterior(showroomPed, barber.Interior)
            setElementDimension(showroomPed, barber.Dimension)

            camera = {}
            camera.view = 'base'
            camera.rotation = 45
            camera.height = -1.5
            camera.zoomLevel = 1
            camera.zoomInterpolate = false
            camera.moveInterpolate = false
            camera.startPos = false
            camera.stopPos = false
            camera.position = {x - 7.5 * math.sin(rz), y + 7.5 * math.cos(rz), z + 2.5, x, y, z}

            fadeCamera(true, 1)
        end, 1000, 1, barber
    )
end

function resetCamera()
    removeEventHandler('onClientPreRender', root, onPreRenderHandler)
    setCameraTarget(localPlayer)
    setElementFrozen(localPlayer, false)

    camera = false
end

-- RENDER FUNCTIONS

function onPreRenderHandler(timeSlice)
    if (not isElement(showroomPed) or not camera) then
        print(1)
        return
    end

    if (getKeyState('mouse1') and not isMTAWindowActive() and not camera.moveInterpolate and not activeButton) then
        local cursorX, cursorY = getCursorPosition()

        if (tonumber(cursorX)) then
            local cursorX = cursorX * screen[1]
            local cursorY = cursorY * screen[2]

            if (not lastCursorPos) then
                lastCursorPos = {
                    onMoveStartX = cursorX,
                    onMoveStartY = cursorY,
                    yawStart = camera.rotation,
                    pitchStart = camera.height
                }
            end

            camera.rotation = lastCursorPos.yawStart - (cursorX - lastCursorPos.onMoveStartX) / screen[1] * 270
            camera.height = lastCursorPos.pitchStart + (cursorY - lastCursorPos.onMoveStartY) / screen[2] * 20

            if (camera.rotation > 360) then
                camera.rotation = camera.rotation - 360
            elseif (camera.rotation < 0) then
                camera.rotation = camera.rotation + 360
            end

            local maxZ = math.abs(getElementDistanceFromCentreOfMassToBaseOfModel(showroomPed) - 1)

            if (camera.height > maxZ) then
                camera.height = maxZ
            elseif (camera.height < -2) then
                camera.height = -2
            end
        end
    elseif (lastCursorPos) then
        lastCursorPos = false
    end

    if (camera.zoomInterpolate) then
        local elapsedTime = getTickCount() - camera.zoomInterpolate[1]
        local progress = elapsedTime / 150

        camera.zoomLevel = interpolateBetween(camera.zoomInterpolate[2], 0, 0, camera.zoomInterpolate[3], 0, 0, progress, 'InOutQuad')

        if (progress >= 1) then
            camera.zoomInterpolate = false
        end
    end

    if (camera.moveInterpolate) then
        local elapsedTime = getTickCount() - camera.moveInterpolate
        local progress = elapsedTime / 475

        camera.rotation, camera.height, camera.zoomLevel = interpolateBetween(
            camera.startPos[1], camera.startPos[2], camera.startPos[3],
            camera.stopPos[1] or camera.startPos[1], camera.stopPos[2] or camera.startPos[2], camera.stopPos[3] or camera.startPos[3],
            progress, 'InOutQuad')

        if (progress >= 1) then
            camera.moveInterpolate = false
            camera.zoomInterpolate = false
        end
    end

    local deltaX = camera.position[1] - camera.position[4]
    local deltaY = camera.position[2] - camera.position[5]
    local theta = math.rad(camera.rotation)

    setCameraMatrix(
        camera.position[4] + deltaX * math.cos(theta) - deltaY * math.sin(theta),
        camera.position[5] + deltaX * math.sin(theta) + deltaY * math.cos(theta),
        camera.position[3] + camera.height,
        camera.position[4],
        camera.position[5],
        camera.position[6],
        0, 70 / camera.zoomLevel
    )
end

function onRenderHandler()
    buttons = {}

    local cursorX, cursorY = getCursorPosition()

    if (tonumber(cursorX)) then
        cursorX = cursorX * screen[1]
        cursorY = cursorY * screen[2]
    end

    if (not tick) then
        tick = getTickCount()
    end

    if (panelState) then
        posX = interpolateBetween(screen[1], 0, 0, 938, 0, 0, (getTickCount() - tick) / 1000, 'OutQuad')
    else
        posX = interpolateBetween(posX, 0, 0, screen[1], 0, 0, (getTickCount() - tick) / 1000, 'OutQuad')
    end

    if (posX == 938) then
        fullScreenMode = true
    else
        fullScreenMode = false
    end

    dxDrawRectangle(x * posX, y * 20, x * 408, y * 50, tocolor(40, 40, 40, 255))
    dxDrawText('Cor do cabelo', x * (posX + 20), y * 20, x * 408, y * (50 + 20), tocolor(255, 255, 255, 255), 1, barber.font[1], 'left', 'center')
    
    dxDrawRectangle(x * posX, y * 70, x * 408, y * 100, tocolor(35, 35, 35, 255))

    dxDrawImage(x * (posX + 75), y * 97, x * 29, y * 47, 'assets/images/arrow.png', 0, 0, 0, (activeButton == 'arrow:down:color' and tocolor(255, 255, 255, 255) or tocolor(94, 94, 94, 255)))
    buttons['arrow:down:color'] = {x * (posX + 75), y * 97, x * 29, y * 47}

    dxDrawImage(x * (posX + 176), y * 92, x * 56, y * 56, 'assets/images/spray.png', 0, 0, 0, tocolor(94, 94, 94, 255))

    dxDrawImage(x * (posX + 300), y * 97, x * 29, y * 47, 'assets/images/arrow.png', 180, 0, 0, (activeButton == 'arrow:up:color' and tocolor(255, 255, 255, 255) or tocolor(94, 94, 94, 255)))
    buttons['arrow:up:color'] = {x * (posX + 300), y * 97, x * 29, y * 47}

    dxDrawRectangle(x * posX, y * 170, x * 408, y * 50, tocolor(40, 40, 40, 255))
    dxDrawText('Tipo do cabelo', x * (posX + 20), y * 170, x * 408, y * (50 + 170), tocolor(255, 255, 255, 255), 1, barber.font[1], 'left', 'center')

    dxDrawRectangle(x * posX, y * 220, x * 408, y * 100, tocolor(35, 35, 35, 255))

    dxDrawImage(x * (posX + 75), y * 247, x * 29, y * 47, 'assets/images/arrow.png', 0, 0, 0, (activeButton == 'arrow:down:type' and tocolor(255, 255, 255, 255) or tocolor(94, 94, 94, 255)))
    buttons['arrow:down:type'] = {x * (posX + 75), y * 247, x * 29, y * 47}

    dxDrawImage(x * (posX + 176), y * 242, x * 56, y * 56, 'assets/images/hair.png', 0, 0, 0, tocolor(94, 94, 94, 255))

    dxDrawImage(x * (posX + 300), y * 247, x * 29, y * 47, 'assets/images/arrow.png', 180, 0, 0, (activeButton == 'arrow:up:type' and tocolor(255, 255, 255, 255) or tocolor(94, 94, 94, 255)))
    buttons['arrow:up:type'] = {x * (posX + 300), y * 247, x * 29, y * 47}

    if (getKeyState('backspace')) then
        if (fullScreenMode and panelState) then
            togglePanel(false)
        end
    end

    activeButton = false

    if (tonumber(cursorX)) then
        for k, v in pairs(buttons) do
            if (cursorX >= v[1] and cursorX <= v[1] + v[3] and cursorY >= v[2] and cursorY <= v[2] + v[4]) then
                activeButton = k
            end
        end
    end
end

-- CLICK FUNCTION

function onClickHandler(button, state)
    if (button == 'left' and state == 'up') then
        if (panelState) then
            if (activeButton) then
                local buttonDetails = split(activeButton, ':')
                if (buttonDetails[1] == 'arrow') then
                    if (buttonDetails[2] == 'up') then
                        if (buttonDetails[3] == 'color') then
                            if (HAIRS.Colors[HAIRS.Types[barber.typeSelected]]) then
                                if (barber.colorSelected + 1 > #HAIRS.Colors[HAIRS.Types[barber.typeSelected]]) then
                                    return false
                                end

                                barber.colorSelected = barber.colorSelected + 1
                                setPedClothes(showroomPed, 1, HAIRS.Colors[HAIRS.Types[barber.typeSelected]][barber.colorSelected])
                            end
                        elseif (buttonDetails[3] == 'type') then
                            if (barber.typeSelected + 1 > #HAIRS.Types) then
                                return false
                            end

                            barber.colorSelected = 0
                            barber.typeSelected = barber.typeSelected + 1
                            setPedClothes(showroomPed, 1, HAIRS.Types[barber.typeSelected])
                        end
                    elseif (buttonDetails[2] == 'down') then
                        if (buttonDetails[3] == 'color') then
                            if (HAIRS.Colors[HAIRS.Types[barber.typeSelected]]) then
                                if (barber.colorSelected - 1 < 1) then
                                    return false
                                end

                                barber.colorSelected = barber.colorSelected - 1
                                setPedClothes(showroomPed, 1, HAIRS.Colors[HAIRS.Types[barber.typeSelected]][barber.colorSelected])
                            end
                        elseif (buttonDetails[3] == 'type') then
                            if (barber.typeSelected - 1 < 1) then
                                return false
                            end

                            barber.colorSelected = 0
                            barber.typeSelected = barber.typeSelected - 1
                            setPedClothes(showroomPed, 1, HAIRS.Types[barber.typeSelected])
                        end
                    end
                end
            end
        end
    end
end

-- KEY FUNCTION

function onKeyHandler(button, press)
    if (press) then
        if (panelState) then
            if (isCursorShowing() and not isMTAWindowActive()) then
                if (button == 'mouse_wheel_up' or button == 'mouse_wheel_down') then
                    if (not activeButton) then
                        if (camera and not camera.zoomInterpolate) then
                            local value = 0

                            if (button == 'mouse_wheel_down') then
                                if (camera.zoomLevel > 0.5) then
                                    value = camera.zoomLevel - 0.2 * camera.zoomLevel
                                end
                            elseif (camera.zoomLevel <= 1.75) then
                                value = camera.zoomLevel + 0.2 * camera.zoomLevel
                            end

                            if (value < 0.5) then
                                value = 0.5
                            elseif (value > 1.75) then
                                value = 1.75
                            end

                            camera.zoomInterpolate = {getTickCount(), camera.zoomLevel, value}
                        end
                    end
                elseif (button == 'enter') then
                    if (getPlayerMoney() >= HAIR_PRICE) then
                        setPedClothes(localPlayer, 1, getPedClothesID(showroomPed, 1))
                        takePlayerMoney(HAIR_PRICE)
                        togglePanel(false)
                        OUTPUT_MESSAGE_CLIENT('Você comprou um Corte de Cabelo por $' .. formatNumber(HAIR_PRICE) .. ' com sucesso.', 'success')
                    else
                        OUTPUT_MESSAGE_CLIENT('Você não possui $' .. formatNumber(HAIR_PRICE) .. ' para comprar esse corte de cabelo!', 'error')
                    end
                end
            end
        end
    end
end

-- TRIGGER EVENTS

addEvent('barbers.showMenu', true)
addEventHandler('barbers.showMenu', root, function(_barberID)
    barberID = _barberID
    togglePanel(true)
end)

-- USEFUL FUNCTIONS

function setPedClothes(thePed, clothingSlot, clothingID)
    local texture, model = getClothesByTypeIndex(clothingSlot, clothingID)

    return addPedClothes(thePed, texture, model, clothingSlot)
end

function getPedClothesID(thePed, clothingSlot)
    local texture, model = getPedClothes(thePed, clothingSlot)

    if (not texture) then
        return false
    else
        local _, id = getTypeIndexFromClothes(texture, model)
        return id
    end
end

function formatNumber(number, stepper)
    local left, center, right = string.match(math.floor(number), '^([^%d]*%d)(%d*)(.-)$')
    return left .. string.reverse(string.gsub(string.reverse(center), '(%d%d%d)', '%1' .. (stepper or '.'))) .. right
end